
import java.awt.*;
import java.io.*;
import java.lang.*;
import java.applet.*;
import java.util.*;
import netscape.javascript.JSObject;

/** 
    <p> This applet saves files onto the user's hard drive. The issues involved with this are the following: </p>
    <ol>
        <li> Does a backup of the original file need to be made? </li>
        <li> Should the user be prompted if the named file exists and they are going to overwrite it?</li>
    </ol>
    <p> CaluMath must save the following as global variables in order for this applet to run.</p>
    <ol>
        <li> CM_FileDataForCaluMathSaveFileApplet. This is text to be written to the file. </li>
        <li> CM_FileDirectoryForCaluMathSaveFileApplet. This is the path to a directory. The written file will be written to a subdirectory of this directory. 
            This should either be a path to the CaluMath directory (that JavaScript obtains from the location property, or the word "userHome" to signify that 
            the Applet should use the userHome property of the System object. In this case, if a Desktop subfolder of the userHome property exists, the file name could be Desktop/MyFile.html to save the file on the Desktop.  </li>
        <li> CM_FileNameForCaluMathSaveFileApplet. This is the path and file name from the CM_FileDirectoryForCaluMathSaveFileApplet Directory. For example, MyWebPages/MyFile.html.  </li>
        <li> CM_FileContainingFolderForCaluMathSaveFileApplet. The path to the folder that contains the file to be saved. It includes CM_FileDirectoryForCaluMathSaveFileApplet plus any subfolders in the path that lead to where the file is saved. </li>
        <li> CM_MakeBackupForCaluMathSaveFileApplet. Either "yes" if a backup is to be made or "no" if a backup is not to be made. </li>
        <li> CM_BackupNameForCaluMathSaveFileApplet. The String to be appended to the filename before the extension which will be used to make the name of the 
        backup file. An example is "_bak". </li>
        <li> CM_PromptUserForFileOverwriteForCaluMathSaveFileApplet. Either "yes" if a prompt should be issued "no" if a file overwrite is automatic. </li>
    </ol>
*/
public class CaluMathSaveFileApplet extends Applet {
    
    DataOutputStream dataOutputStream;
    String continueWithSave = "yes";
    String successfulSave = "yes";
    
    public String myFile = "";
    public String myData = "";
    public String myDirectory = "" ;
    public File myContainingFolder = new File(myFile);
    public String myContainingFolderList = "";
    public File savedFile = new File(myFile);
    public String userPath = "";
    
    public String makeBackup = "";
    public String myBackupName = "";
    public String actualBackupName = "";
    
    public String promptUser = "";
    
    public JSObject win;
    public JSObject JSFileName ;
    public JSObject JSFileData ; 
    
    public String osName ;
    public String userName;
    public String userHome;
    public String userDir;
    
    public Boolean fileExists=true;
    
    public void init(){
        JSObject win=JSObject.getWindow(this);
        makeBackup=win.getMember("CM_MakeBackupForCaluMathSaveFileApplet").toString();
        promptUser=win.getMember("CM_PromptUserForFileOverwriteForCaluMathSaveFileApplet").toString();
        myFile=win.getMember("CM_FileNameForCaluMathSaveFileApplet").toString();
        myDirectory=win.getMember("CM_FileDirectoryForCaluMathSaveFileApplet").toString();
        myData = win.getMember("CM_FileDataForCaluMathSaveFileApplet").toString();
        String myContainingFolderStringName = win.getMember("CM_FileContainingFolderForCaluMathSaveFileApplet").toString();
        myContainingFolder = new File(myContainingFolderStringName);
        
        if(myContainingFolder.isDirectory()){
            String stringList[] = myContainingFolder.list();
            for( int i=0; i< stringList.length-1; i++){
                myContainingFolderList= myContainingFolderList + stringList[i]+" , ";
            }
            myContainingFolderList= myContainingFolderList + stringList[stringList.length-1];
        }
        else{
            myContainingFolderList = "Sorry, a list of the files in this folder could not be obtained.";
        }
        
        osName = System.getProperty("os.name","");
        userName = System.getProperty("user.name","");
        userHome = System.getProperty("user.home","");
        userDir = System.getProperty("user.dir","");
        
        if(myDirectory.equals("userHome")){
            myDirectory = userHome;
        }
        
        try{
            if(makeBackup.equals("yes")){
                // We make a backup of the file. This should happen, for example, the first time the file is to be saved.
                // We make a copy of it before saving, which overwrites the original file. 
                myBackupName =win.getMember("CM_BackupNameForCaluMathSaveFileApplet").toString();
                actualBackupName = myDirectory +File.separator + myBackupName;
                InputStream in = new FileInputStream(myDirectory +File.separator + myFile);
                OutputStream out = new FileOutputStream(actualBackupName);   
                byte[] buf = new byte[1024];
                int len;
                while ((len = in.read(buf)) > 0) {
                    out.write(buf, 0, len);
                }
                in.close();
                out.close();
            }
            File CheckExistsFile = new File(myDirectory +File.separator + myFile);
            if(CheckExistsFile.exists()==true && promptUser.equals("yes")){
                //This will open a prompt window and set continueWithSave to no if the prompt returns no;
                continueWithSave = win.eval("CM_PromptUserForFileOverwriteForCaluMathSaveFileAppletFunction()").toString();
            }
            if(continueWithSave.equals("yes")){
                dataOutputStream = new DataOutputStream(new BufferedOutputStream(new FileOutputStream( myDirectory +File.separator + myFile),128));
                dataOutputStream.writeBytes(myData);
                dataOutputStream.flush();
                dataOutputStream.close();
                successfulSave = "yes"; 
                savedFile = new File(myDirectory +File.separator + myFile);
                userPath = savedFile.getAbsolutePath();
                win.eval("CM_AfterSaveFileForCaluMathSaveFileAppletFunction()");
            }
            else{
                win.eval("CM_AfterPromptForFileOverwriteCancelledForCaluMathSaveFileAppletFunction()");
                successfulSave = "cancelled"; 
            }
        }
        catch (SecurityException e) {
            successfulSave = "SecurityException";
            win.eval("CM_AfterSaveFileSecutityExceptionForCaluMathSaveFileAppletFunction()");
        }
        catch (IOException ioe) {
            successfulSave = "IOException";
            win.eval("CM_AfterSaveFileIOExceptionForCaluMathSaveFileAppletFunction()");
        }
    }

  
    public void paint(Graphics g) {
        if(successfulSave =="yes"){
            g.drawString("Saved " + myFile , 10, 10);
            g.drawString("in folder", 10, 30);
            g.drawString(myDirectory,10,50);
        }
        else if(successfulSave =="cancelled"){
            g.drawString("Cancelled saving " + myFile , 10, 10);
            g.drawString("in folder", 10, 30);
            g.drawString(myDirectory,10,50);
        }
        else if(successfulSave ==  "SecurityException") {
            g.drawString("THE FILE WAS NOT SAVED. " , 10, 10);
            g.drawString("PROBABLY YOUR BROWSER'S SECURITY SETTINGS WILL NOT ALLOW YOU"  , 10, 30);
            g.drawString("TO SAVE FILES USING A JAVA APPLET." , 10, 50);
            g.drawString("PLEASE SAVE THE DISPLAYED SOURCE CODE INSTEAD.", 10, 70);
        }
        else if (successfulSave == "IOException") {
            g.drawString("THE FILE WAS NOT SAVED. MOST LIKELY, YOUR FILE NAME WAS INVALID, " , 10, 10);
            g.drawString("OR THE FOLDER IN WHICH YOU TRIED TO SAVE THE FILE DOES NOT EXIST."  , 10, 30);
            g.drawString("THE DETAILS ARE: JAVA IOException  ", 10, 50);
            g.drawString("File "+myFile+ " could not be saved in ", 10, 70);
            g.drawString(myDirectory, 10 ,90);
        }
    }
    
    public static void main(String args[]) {
        CaluMathSaveFileApplet	CaluMathSaveFileApplet1 = new CaluMathSaveFileApplet();
        CaluMathSaveFileApplet1.init();
    }

}
